@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007-2010 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2011 B.A.T.M.A.N. contributors:
  *
- * Andreas Langer <a.langer@q-dsl.de>, Marek Lindner <lindner_marek@yahoo.de>
+ * Andreas Langer <an.langer@gmx.de>, Marek Lindner <lindner_marek@yahoo.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -21,6 +21,8 @@
 
 
 
-#define SOURCE_VERSION "2010.0.0"
+#define SOURCE_VERSION "2011.1.0"
 
-#define BAT_DEVICE "/dev/batman-adv"
+#define SOCKET_PATH_FMT "%s/batman_adv/%s/socket"
+
+#define __packed __attribute((packed))   /* linux kernel compat */
