@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2010 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2009-2011 B.A.T.M.A.N. contributors:
  *
  * Marek Lindner <lindner_marek@yahoo.de>
  *
@@ -20,29 +20,39 @@
  */
 
 
-#define SYS_MODULE_PATH "/sys/module/batman_adv/"
-#define SYS_BATIF_PATH "/sys/class/net/bat0/mesh/"
-#define SYS_LOG_LEVEL "parameters/debug"
+#define SYS_BATIF_PATH_FMT "/sys/class/net/%s/mesh/"
+#define SYS_LOG_LEVEL "log_level"
 #define SYS_LOG "log"
-#define SYS_ORIGINATORS "originators"
-#define SYS_TRANSTABLE_LOCAL "transtable_local"
-#define SYS_TRANSTABLE_GLOBAL "transtable_global"
-#define SYS_AGGR "aggregate_ogm"
+#define SYS_AGGR "aggregated_ogms"
+#define SYS_BONDING "bonding"
+#define SYS_GW_MODE "gw_mode"
+#define SYS_GW_SEL "gw_sel_class"
+#define SYS_GW_BW "gw_bandwidth"
 #define SYS_VIS_MODE "vis_mode"
-#define SYS_VIS_DATA "vis_data"
 #define SYS_ORIG_INTERVAL "orig_interval"
 #define SYS_IFACE_PATH "/sys/class/net"
 #define SYS_MESH_IFACE_FMT SYS_IFACE_PATH"/%s/batman_adv/mesh_iface"
 #define SYS_IFACE_STATUS_FMT SYS_IFACE_PATH"/%s/batman_adv/iface_status"
+#define SYS_FRAG "fragmentation"
+
+enum gw_modes {
+	GW_MODE_OFF,
+	GW_MODE_CLIENT,
+	GW_MODE_SERVER,
+};
+
+extern const char *sysfs_param_enable[];
+extern const char *sysfs_param_server[];
 
-void originators_usage(void);
-void trans_local_usage(void);
-void trans_global_usage(void);
 void aggregation_usage(void);
+void bonding_usage(void);
+void fragmentation_usage(void);
+void gw_mode_usage(void);
 void vis_mode_usage(void);
 void orig_interval_usage(void);
-int log_print(int argc, char **argv);
-int interface(int argc, char **argv);
-int handle_loglevel(int argc, char **argv);
-int handle_sys_table(int argc, char **argv, char *file_path, void table_usage(void));
-int handle_sys_setting(int argc, char **argv, char *file_path, void setting_usage(void));
+int interface(char *mesh_iface, int argc, char **argv);
+int handle_loglevel(char *mesh_iface, int argc, char **argv);
+int handle_sys_setting(char *mesh_iface, int argc, char **argv,
+		       char *file_path, void setting_usage(void),
+		       const char *sysfs_param[]);
+int handle_gw_setting(char *mesh_iface, int argc, char **argv);
