@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007-2010 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2011 B.A.T.M.A.N. contributors:
  *
- * Andreas Langer <a.langer@q-dsl.de>
+ * Andreas Langer <an.langer@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -29,6 +29,7 @@
 #include <sys/time.h>
 #include <arpa/inet.h>
 #include <net/if.h>
+#include <net/if_arp.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
@@ -50,8 +51,12 @@ if ((size_t)(buff_len) < (check_len)) { \
 	return; \
 }
 
+static unsigned short dump_level = DUMP_TYPE_BATOGM | DUMP_TYPE_BATICMP | DUMP_TYPE_BATUCAST |
+		DUMP_TYPE_BATBCAST | DUMP_TYPE_BATVIS | DUMP_TYPE_BATFRAG | DUMP_TYPE_NONBAT;
 
-void tcpdump_usage(void)
+static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed);
+
+static void tcpdump_usage(void)
 {
 	printf("Usage: batctl tcpdump [options] interface [interface]\n");
 	printf("options:\n");
@@ -63,11 +68,12 @@ void tcpdump_usage(void)
 	printf(" \t\t%d - batman unicast packets\n", DUMP_TYPE_BATUCAST);
 	printf(" \t\t%d - batman broadcast packets\n", DUMP_TYPE_BATBCAST);
 	printf(" \t\t%d - batman vis packets\n", DUMP_TYPE_BATVIS);
+	printf(" \t\t%d - batman fragmented packets\n", DUMP_TYPE_BATFRAG);
 	printf(" \t\t%d - non batman packets\n", DUMP_TYPE_NONBAT);
 	printf(" \t\t%d - batman ogm & non batman packets\n", DUMP_TYPE_BATOGM | DUMP_TYPE_NONBAT);
 }
 
-void print_time(void)
+static int print_time(void)
 {
 	struct timeval tv;
 	struct tm *tm;
@@ -76,9 +82,10 @@ void print_time(void)
 	tm = localtime(&tv.tv_sec);
 
 	printf("%02d:%02d:%02d.%06ld ", tm->tm_hour, tm->tm_min, tm->tm_sec, tv.tv_usec);
+	return 1;
 }
 
-void dump_arp(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
+static void dump_arp(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
 {
 	struct ether_arp *arphdr;
 
@@ -105,7 +112,7 @@ void dump_arp(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
 	}
 }
 
-void dump_ip(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
+static void dump_ip(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
 {
 	struct iphdr *iphdr, *tmp_iphdr;
 	struct tcphdr *tcphdr;
@@ -221,35 +228,26 @@ void dump_ip(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
 	}
 }
 
-void dump_vlan(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
+static void dump_vlan(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct vlanhdr *vlanhdr;
 
-	vlanhdr = (struct vlanhdr *)packet_buff;
-	LEN_CHECK((size_t)buff_len, sizeof(struct vlanhdr), "VLAN");
+	vlanhdr = (struct vlanhdr *)(packet_buff + sizeof(struct ether_header));
+	LEN_CHECK((size_t)buff_len, sizeof(struct ether_header) + sizeof(struct vlanhdr), "VLAN");
 
 	if (!time_printed)
-		print_time();
+		time_printed = print_time();
 
 	vlanhdr->vid = ntohs(vlanhdr->vid);
 	printf("vlan %u, p %u, ", vlanhdr->vid, vlanhdr->vid >> 12);
 
-	switch (ntohs(vlanhdr->ether_type)) {
-	case ETH_P_ARP:
-		dump_arp(packet_buff + sizeof(struct vlanhdr),
-			 buff_len - sizeof(struct vlanhdr), 1);
-		break;
-	case ETH_P_IP:
-		dump_ip(packet_buff + sizeof(struct vlanhdr),
-			buff_len - sizeof(struct vlanhdr), 1);
-		break;
-	default:
-		printf(" unknown payload ether type: 0x%04x\n", ntohs(vlanhdr->ether_type));
-		break;
-	}
+	/* overwrite vlan tags */
+	memmove(packet_buff + 4, packet_buff, 2 * ETH_ALEN);
+
+	parse_eth_hdr(packet_buff + 4, buff_len - 4, read_opt, time_printed);
 }
 
-void dump_batman_ogm(unsigned char *packet_buff, ssize_t buff_len, int read_opt)
+static void dump_batman_ogm(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
 	struct batman_packet *batman_packet;
@@ -259,32 +257,34 @@ void dump_batman_ogm(unsigned char *packet_buff, ssize_t buff_len, int read_opt)
 	ether_header = (struct ether_header *)packet_buff;
 	batman_packet = (struct batman_packet *)(packet_buff + sizeof(struct ether_header));
 
-	print_time();
+	if (!time_printed)
+		print_time();
 
 	printf("BAT %s: ",
 	       get_name_by_macaddr((struct ether_addr *)batman_packet->orig, read_opt));
 
-	printf("OGM via neigh %s, seqno %d, tq %3d, ttl %2d, v %d, flags [%c%c], length %zu\n",
+	printf("OGM via neigh %s, seq %u, tq %3d, ttl %2d, v %d, flags [%c%c%c%c], length %zu\n",
 	        get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt),
-	        ntohs(batman_packet->seqno), batman_packet->tq,
+	        ntohl(batman_packet->seqno), batman_packet->tq,
 	        batman_packet->ttl, batman_packet->version,
-	        (batman_packet->flags & VIS_SERVER ? 'V' : '.'),
 	        (batman_packet->flags & DIRECTLINK ? 'D' : '.'),
+	        (batman_packet->flags & VIS_SERVER ? 'V' : '.'),
+	        (batman_packet->flags & PRIMARIES_FIRST_HOP ? 'F' : '.'),
+	        (batman_packet->gw_flags ? 'G' : '.'),
 	        (size_t)buff_len - sizeof(struct ether_header));
 }
 
-void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int read_opt)
+static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
-	struct ether_header *ether_header;
 	struct icmp_packet *icmp_packet;
 	char *name;
 
 	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct icmp_packet), "BAT ICMP");
 
-	ether_header = (struct ether_header *)packet_buff;
 	icmp_packet = (struct icmp_packet *)(packet_buff + sizeof(struct ether_header));
 
-	print_time();
+	if (!time_printed)
+		print_time();
 
 	printf("BAT %s > ", get_name_by_macaddr((struct ether_addr *)icmp_packet->orig, read_opt));
 
@@ -316,7 +316,7 @@ void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int read_opt
 	}
 }
 
-void dump_batman_ucast(unsigned char *packet_buff, ssize_t buff_len, int read_opt)
+static void dump_batman_ucast(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
 	struct unicast_packet *unicast_packet;
@@ -328,37 +328,22 @@ void dump_batman_ucast(unsigned char *packet_buff, ssize_t buff_len, int read_op
 	ether_header = (struct ether_header *)packet_buff;
 	unicast_packet = (struct unicast_packet *)(packet_buff + sizeof(struct ether_header));
 
-	print_time();
+	if (!time_printed)
+		time_printed = print_time();
 
 	printf("BAT %s > ",
 	       get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt));
 
-	printf("%s: UCAST, ttl %hu, ",
+	printf("%s: UCAST, ttl %hhu, ",
 	       get_name_by_macaddr((struct ether_addr *)unicast_packet->dest, read_opt),
 	       unicast_packet->ttl);
 
-	ether_header = (struct ether_header *)(packet_buff + sizeof(struct ether_header) + sizeof(struct unicast_packet));
-
-	switch (ntohs(ether_header->ether_type)) {
-	case ETH_P_ARP:
-		dump_arp(packet_buff + (2 * sizeof(struct ether_header)) + sizeof(struct unicast_packet),
-			buff_len - (2 * sizeof(struct ether_header)) - sizeof(struct unicast_packet), 1);
-		break;
-	case ETH_P_IP:
-		dump_ip(packet_buff + (2 * sizeof(struct ether_header)) + sizeof(struct unicast_packet),
-			buff_len - (2 * sizeof(struct ether_header)) - sizeof(struct unicast_packet), 1);
-		break;
-	case ETH_P_8021Q:
-		dump_vlan(packet_buff + (2 * sizeof(struct ether_header)) + sizeof(struct unicast_packet),
-			  buff_len - (2 * sizeof(struct ether_header)) - sizeof(struct unicast_packet), 1);
-		break;
-	default:
-		printf(" unknown payload ether type: 0x%04x\n", ntohs(ether_header->ether_type));
-		break;
-	}
+	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct unicast_packet),
+		      buff_len - ETH_HLEN - sizeof(struct unicast_packet),
+		      read_opt, time_printed);
 }
 
-void dump_batman_bcast(unsigned char *packet_buff, ssize_t buff_len, int read_opt)
+static void dump_batman_bcast(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
 	struct bcast_packet *bcast_packet;
@@ -370,50 +355,210 @@ void dump_batman_bcast(unsigned char *packet_buff, ssize_t buff_len, int read_op
 	ether_header = (struct ether_header *)packet_buff;
 	bcast_packet = (struct bcast_packet *)(packet_buff + sizeof(struct ether_header));
 
-	print_time();
+	if (!time_printed)
+		time_printed = print_time();
 
 	printf("BAT %s: ",
 	       get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt));
 
-	printf("BCAST, orig %s, seqno %hu, ",
+	printf("BCAST, orig %s, seq %u, ",
 	       get_name_by_macaddr((struct ether_addr *)bcast_packet->orig, read_opt),
-	       ntohs(bcast_packet->seqno));
+	       ntohl(bcast_packet->seqno));
 
-	ether_header = (struct ether_header *)(packet_buff + sizeof(struct ether_header) + sizeof(struct bcast_packet));
+	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct bcast_packet),
+		      buff_len - ETH_HLEN - sizeof(struct bcast_packet),
+		      read_opt, time_printed);
+}
 
-	switch (ntohs(ether_header->ether_type)) {
+static void dump_batman_frag(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
+{
+	struct unicast_frag_packet *unicast_frag_packet;
+
+	LEN_CHECK((size_t)buff_len - ETH_HLEN, sizeof(struct unicast_frag_packet), "BAT FRAG");
+	LEN_CHECK((size_t)buff_len - ETH_HLEN - sizeof(struct unicast_frag_packet), (size_t)ETH_HLEN, "BAT FRAG (unpacked)");
+
+	unicast_frag_packet = (struct unicast_frag_packet *)(packet_buff + ETH_HLEN);
+
+	if (!time_printed)
+		time_printed = print_time();
+
+	printf("BAT %s > ",
+	       get_name_by_macaddr((struct ether_addr *)unicast_frag_packet->orig, read_opt));
+
+	printf("%s: FRAG, seq %hu, ttl %hhu, flags [%c%c], ",
+	       get_name_by_macaddr((struct ether_addr *)unicast_frag_packet->dest, read_opt),
+	       ntohs(unicast_frag_packet->seqno), unicast_frag_packet->ttl,
+	       (unicast_frag_packet->flags & UNI_FRAG_HEAD ? 'H' : '.'),
+	       (unicast_frag_packet->flags & UNI_FRAG_LARGETAIL ? 'L' : '.'));
+
+	if (unicast_frag_packet->flags & UNI_FRAG_HEAD)
+		parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct unicast_frag_packet),
+			      buff_len - ETH_HLEN - sizeof(struct unicast_frag_packet),
+			      read_opt, time_printed);
+	else
+		printf("length %zu\n", (size_t)buff_len - ETH_HLEN - sizeof(struct unicast_frag_packet));
+}
+
+static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
+{
+	struct batman_packet *batman_packet;
+	struct ether_header *eth_hdr;
+
+	eth_hdr = (struct ether_header *)packet_buff;
+
+	switch (ntohs(eth_hdr->ether_type)) {
 	case ETH_P_ARP:
-		dump_arp(packet_buff + (2 * sizeof(struct ether_header)) + sizeof(struct bcast_packet),
-			buff_len - (2 * sizeof(struct ether_header)) - sizeof(struct bcast_packet), 1);
+		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
+			dump_arp(packet_buff + ETH_HLEN, buff_len - ETH_HLEN, time_printed);
 		break;
 	case ETH_P_IP:
-		dump_ip(packet_buff + (2 * sizeof(struct ether_header)) + sizeof(struct bcast_packet),
-			buff_len - (2 * sizeof(struct ether_header)) - sizeof(struct bcast_packet), 1);
+		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
+			dump_ip(packet_buff + ETH_HLEN, buff_len - ETH_HLEN, time_printed);
 		break;
 	case ETH_P_8021Q:
-		dump_vlan(packet_buff + (2 * sizeof(struct ether_header)) + sizeof(struct bcast_packet),
-			  buff_len - (2 * sizeof(struct ether_header)) - sizeof(struct bcast_packet), 1);
+		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
+			dump_vlan(packet_buff, buff_len, read_opt, time_printed);
 		break;
+	case ETH_P_BATMAN:
+		batman_packet = (struct batman_packet *)(packet_buff + ETH_HLEN);
+
+		switch (batman_packet->packet_type) {
+		case BAT_PACKET:
+			if (dump_level & DUMP_TYPE_BATOGM)
+				dump_batman_ogm(packet_buff, buff_len, read_opt, time_printed);
+			break;
+		case BAT_ICMP:
+			if (dump_level & DUMP_TYPE_BATICMP)
+				dump_batman_icmp(packet_buff, buff_len, read_opt, time_printed);
+			break;
+		case BAT_UNICAST:
+			if (dump_level & DUMP_TYPE_BATUCAST)
+				dump_batman_ucast(packet_buff, buff_len, read_opt, time_printed);
+			break;
+		case BAT_BCAST:
+			if (dump_level & DUMP_TYPE_BATBCAST)
+				dump_batman_bcast(packet_buff, buff_len, read_opt, time_printed);
+			break;
+		case BAT_VIS:
+			if (dump_level & DUMP_TYPE_BATVIS)
+				printf("Warning - batman vis packet received: function not implemented yet\n");
+			break;
+		case BAT_UNICAST_FRAG:
+			if (dump_level & DUMP_TYPE_BATFRAG)
+				dump_batman_frag(packet_buff, buff_len, read_opt, time_printed);
+			break;
+		default:
+			printf("Warning - packet contains unknown batman packet type: 0x%02x\n", batman_packet->packet_type);
+			break;
+		}
+
+		break;
+
 	default:
-		printf(" unknown payload ether type: 0x%04x\n", ntohs(ether_header->ether_type));
+		printf("Warning - packet contains unknown ether type: 0x%04x\n", ntohs(eth_hdr->ether_type));
 		break;
 	}
 }
 
+static int monitor_header_length(unsigned char *packet_buff, ssize_t buff_len, int32_t hw_type)
+{
+	struct radiotap_header *radiotap_hdr;
+	switch (hw_type) {
+	case ARPHRD_IEEE80211_PRISM:
+		if (buff_len <= (ssize_t)PRISM_HEADER_LEN)
+			return -1;
+		else
+			return PRISM_HEADER_LEN;
+
+	case ARPHRD_IEEE80211_RADIOTAP:
+		if (buff_len <= (ssize_t)RADIOTAP_HEADER_LEN)
+			return -1;
+
+		radiotap_hdr = (struct radiotap_header*)packet_buff;
+		if (buff_len <= radiotap_hdr->it_len)
+			return -1;
+		else
+			return radiotap_hdr->it_len;
+	}
+
+	return -1;
+}
+
+static void parse_wifi_hdr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
+{
+	struct ether_header *eth_hdr;
+	struct ieee80211_hdr *wifi_hdr;
+	unsigned char *shost, *dhost;
+	uint16_t fc;
+	int hdr_len;
+
+	/* we assume a minimum size of 38 bytes
+	 * (802.11 data frame + LLC)
+	 * before we calculate the real size */
+	if (buff_len <= 38)
+		return;
+
+	wifi_hdr = (struct ieee80211_hdr *)packet_buff;
+	fc = ntohs(wifi_hdr->frame_control);
+
+	/* not carrying payload */
+	if ((fc & IEEE80211_FCTL_FTYPE) != IEEE80211_FTYPE_DATA)
+		return;
+
+	/* encrypted packet */
+	if (fc & IEEE80211_FCTL_PROTECTED)
+		return;
+
+	shost = wifi_hdr->addr2;
+	if (fc & IEEE80211_FCTL_FROMDS)
+		shost = wifi_hdr->addr3;
+	else if (fc & IEEE80211_FCTL_TODS)
+		shost = wifi_hdr->addr4;
+
+	dhost = wifi_hdr->addr1;
+	if (fc & IEEE80211_FCTL_TODS)
+		dhost = wifi_hdr->addr3;
+
+	hdr_len = 24;
+	if ((fc & IEEE80211_FCTL_FROMDS) && (fc & IEEE80211_FCTL_TODS))
+		hdr_len = 30;
+
+	if (fc & IEEE80211_STYPE_QOS_DATA)
+		hdr_len += 2;
+
+	/* LLC */
+	hdr_len += 8;
+	hdr_len -= sizeof(struct ether_header);
+
+	if (buff_len <= hdr_len)
+		return;
+
+	buff_len -= hdr_len;
+	packet_buff += hdr_len;
+
+	eth_hdr = (struct ether_header *)packet_buff;
+	memmove(eth_hdr->ether_shost, shost, ETH_ALEN);
+	memmove(eth_hdr->ether_dhost, dhost, ETH_ALEN);
+
+	 /* printf("parse_wifi_hdr(): ether_type: 0x%04x\n", ntohs(eth_hdr->ether_type));
+	printf("parse_wifi_hdr(): shost: %s\n", ether_ntoa_long((struct ether_addr *)eth_hdr->ether_shost));
+	printf("parse_wifi_hdr(): dhost: %s\n", ether_ntoa_long((struct ether_addr *)eth_hdr->ether_dhost)); */
+
+	parse_eth_hdr(packet_buff, buff_len, read_opt, time_printed);
+}
+
 int tcpdump(int argc, char **argv)
 {
 	struct ifreq req;
 	struct timeval tv;
 	struct dump_if *dump_if, *dump_if_tmp;
-	struct batman_packet *batman_packet;
 	struct list_head_first dump_if_list;
 	fd_set wait_sockets, tmp_wait_sockets;
 	ssize_t read_len;
-	int ret = EXIT_FAILURE, res, optchar, found_args = 1, max_sock = 0, ether_type, tmp;
+	int ret = EXIT_FAILURE, res, optchar, found_args = 1, max_sock = 0, tmp;
 	int read_opt = USE_BAT_HOSTS;
-	unsigned char dump_level = DUMP_TYPE_BATOGM | DUMP_TYPE_BATICMP |
-		DUMP_TYPE_BATUCAST | DUMP_TYPE_BATBCAST | DUMP_TYPE_BATVIS | DUMP_TYPE_NONBAT;
 	unsigned char packet_buff[2000];
+	int monitor_header_len = -1;
 
 	while ((optchar = getopt(argc, argv, "hnp:")) != -1) {
 		switch (optchar) {
@@ -442,7 +587,7 @@ int tcpdump(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	bat_hosts_init();
+	bat_hosts_init(read_opt);
 
 	/* init interfaces list */
 	INIT_LIST_HEAD_FIRST(dump_if_list);
@@ -471,6 +616,28 @@ int tcpdump(int argc, char **argv)
 		memset(&req, 0, sizeof (struct ifreq));
 		strncpy(req.ifr_name, dump_if->dev, IFNAMSIZ);
 
+		res = ioctl(dump_if->raw_sock, SIOCGIFHWADDR, &req);
+		if (res < 0) {
+			printf("Error - can't create raw socket (SIOCGIFHWADDR): %s\n", strerror(errno));
+			close(dump_if->raw_sock);
+			goto out;
+		}
+
+		dump_if->hw_type = req.ifr_hwaddr.sa_family;
+
+		switch (dump_if->hw_type) {
+		case ARPHRD_ETHER:
+		case ARPHRD_IEEE80211_PRISM:
+		case ARPHRD_IEEE80211_RADIOTAP:
+			break;
+		default:
+			printf("Error - interface '%s' is of unknown type: %i\n", dump_if->dev, dump_if->hw_type);
+			goto out;
+		}
+
+		memset(&req, 0, sizeof (struct ifreq));
+		strncpy(req.ifr_name, dump_if->dev, IFNAMSIZ);
+
 		res = ioctl(dump_if->raw_sock, SIOCGIFINDEX, &req);
 
 		if (res < 0) {
@@ -533,53 +700,18 @@ int tcpdump(int argc, char **argv)
 				continue;
 			}
 
-			ether_type = ntohs(((struct ether_header *)packet_buff)->ether_type);
-
-			switch (ether_type) {
-			case ETH_P_ARP:
-				if (dump_level & DUMP_TYPE_NONBAT)
-					dump_arp(packet_buff + sizeof(struct ether_header),
-						read_len - sizeof(struct ether_header), 0);
+			switch (dump_if->hw_type) {
+			case ARPHRD_ETHER:
+				parse_eth_hdr(packet_buff, read_len, read_opt, 0);
 				break;
-			case ETH_P_IP:
-				if (dump_level & DUMP_TYPE_NONBAT)
-					dump_ip(packet_buff + sizeof(struct ether_header),
-						read_len - sizeof(struct ether_header), 0);
-				break;
-			case ETH_P_8021Q:
-				if (dump_level & DUMP_TYPE_NONBAT)
-					dump_vlan(packet_buff + sizeof(struct ether_header),
-						read_len - sizeof(struct ether_header), 0);
-				break;
-			case ETH_P_BATMAN:
-				batman_packet = (struct batman_packet *)(packet_buff + sizeof(struct ether_header));
-
-				switch (batman_packet->packet_type) {
-				case BAT_PACKET:
-					if (dump_level & DUMP_TYPE_BATOGM)
-						dump_batman_ogm(packet_buff, read_len, read_opt);
-					break;
-				case BAT_ICMP:
-					if (dump_level & DUMP_TYPE_BATICMP)
-						dump_batman_icmp(packet_buff, read_len, read_opt);
-					break;
-				case BAT_UNICAST:
-					if (dump_level & DUMP_TYPE_BATUCAST)
-						dump_batman_ucast(packet_buff, read_len, read_opt);
-					break;
-				case BAT_BCAST:
-					if (dump_level & DUMP_TYPE_BATBCAST)
-						dump_batman_bcast(packet_buff, read_len, read_opt);
-					break;
-				case BAT_VIS:
-					if (dump_level & DUMP_TYPE_BATVIS)
-						printf("Warning - batman vis packet received: function not implemented yet\n");
-					break;
-				}
-
+			case ARPHRD_IEEE80211_PRISM:
+			case ARPHRD_IEEE80211_RADIOTAP:
+				monitor_header_len = monitor_header_length(packet_buff, read_len, dump_if->hw_type);
+				if (monitor_header_len >= 0)
+					parse_wifi_hdr(packet_buff + monitor_header_len, read_len - monitor_header_len, read_opt, 0);
 				break;
 			default:
-				printf("Warning - packet contains unknown ether type: 0x%04x\n", ether_type);
+				/* should not happen */
 				break;
 			}
 
